#!/usr/bin/perl
#
# $Id: look4scans.pl
#
#  Script to scan a set of tcpdump compressed files for scans.
#
#  Adapted from scripts written by Stephen Northcutt.
#  
#  Written by Bill Ralph 18 Sep 1998
#
# Set up some variables.
#
use Getopt::Long;
use POSIX qw(strftime);
use Socket;
#
$CID_PATH = "/usr/local/logger";
$ENV{PATH} = "/bin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:$CID_PATH";
@site_list = ("Site1", "Site2", "Site3");
#
sub usage {
        print "Usage: look4scans.pl [-d DATE] -f FILTER.\n";
	exit 2;
}
#
#	Parse the parameters.
#
#
&GetOptions("d:s", \$snifdate, "f=s", \$filter);
##
#
#  Check parameter validity.
#
if ($filter eq "")
{
	usage();
}
#
# If no snifdate specified on command line, assume today.
#
if ($snifdate eq "") {
   $snifdate = strftime("%y%m%d", localtime);
}
#
#  Break up the "snifdate" into its useful components.
#
$year=substr($snifdate,0,2);
$mon=substr($snifdate,2,2) - 1;
$mday=substr($snifdate,4,2);
$subdir=strftime("%h%d", 0, 0, 0, $mday, $mon, $year, 0, 0, 0);
#
#
# Open the sort pipline into which the raw data is flushed.
#
$sort_cmd = "sort | uniq -u | sort >/tmp/sort_$$";
open(RESULT, "|$sort_cmd") or die ("Unable to open sort pipeline.");
#
#
# Start a master loop to cycle through the Navy sites.
#
foreach $site (@site_list)
{
#
# Once the Site is identified from the command line,
# load the needed external Parameters.
#
   unshift(@INC, "$CID_PATH/sites");
   require "${site}.ph";

#
#
# Construct the pattern.
#
   $net_pattern = "(dst net $SITE_IP[0]";
   if (scalar(@SITE_IP) > 1) {
      foreach $net (@SITE_IP[1..$#SITE_IP]) {
         $net_pattern .= " or dst net $net";
      }
   }
   $net_pattern .= ")";
#  print STDOUT "Net pattern: $net_pattern\n";
#
   $pattern = "(";
   open(FILTER, "<$GENERIC_FILTERS/$filter") or die ("No such filter: $filter");
   while (<FILTER>) {
      $pattern .= $_;
   }
   close(FILTER);
   chomp($pattern);
   $pattern .= ") and $net_pattern";
   $pattern = '"' . $pattern . '"';
#  print STDOUT "Pattern: $pattern\n";
#
# Ready to start the work now, find the compressed data files.
#
# 
   chdir("${ANALYZER_DIR}/${subdir}");
   @files=<tcp.*.gz>;
   if ($files[0] eq "") {
#
#  No hourly files, they must have been already condensed into daily ones.
#
#   @files = <day_*.gz>;
    @files=("day_tcp.gz");
}
   foreach $file (@files) {
      $sys_cmd = "gunzip -c $file | tcpdump -n -r - $pattern";
      open(RAWFILE, "$sys_cmd|") or die("Can't open raw data file.");
      while (<RAWFILE>) {
#
# Remove those lines TCPDUMP reports as "truncated data."
#
         next if (/truncat/);
         ($crap, $crap1, $junk3, $crap2, $flags) = split(/\s+/,$_);
         chop($crap2);
#
# If the time field is null, skip to next record.
#
         ($time, $milli_time) = split(/\./,$crap);
         next if (not $time); 
         ($part1, $part2, $part3, $part4, $src_port) = split(/\./, $crap1);
         $src="$part1.$part2.$part3.$part4";
         ($part1, $part2, $part3, $part4, $dst_port) = split(/\./, $crap2);
         $dest="$part1.$part2.$part3.$part4";
         print RESULT "$src > $dest :$flags\n";
      }
      close(RAWFILE);
   }
#
# End of the site loop
#
}
close(RESULT);
#
open(RESULT, "</tmp/sort_$$");
#
$oldsrcip = "0.0.0.0";
$olddataline = "gobbledeegook";
while (<RESULT>)
{
    $dataline = $_;
    @fields = split(/\s+/,$dataline);
    @srcoctets = split(/\./,$fields[0]);
    $srcip = "$srcoctets[0].$srcoctets[1].$srcoctets[2].$srcoctets[3]";

    if ($ip_name{$srcip}) {
       $srcname = $ip_name{$srcip};
    } else {
       $binip = pack "c4",$srcoctets[0],$srcoctets[1],$srcoctets[2],$srcoctets[3];
       @info = gethostbyaddr($binip,AF_INET);
       $srcname = $info[0];
       $ip_name{$srcip} =  $srcname;
    }
    if ($srcip eq $oldsrcip)
    {
       if ($dataline ne $olddataline)
       {
           print STDOUT $dataline;
           $olddataline = $dataline;
       }
    }
    else
    {
       print STDOUT "\n";
       print STDOUT "$srcip : $srcname\n";
       print STDOUT $dataline;
       $oldsrcip = $srcip;
       $olddataline = $dataline;
    }
}
close(RESULT);
unlink("/tmp/sort_$$");
