#!/usr/bin/perl
#
# startup-wrapper for the tcpdump used in the CID tool.
#
# Author: Olav Kolbu <olav.kolbu@usit.uio.no>
# Modifier: Bill Ralph <wralph@nswc.navy.mil>
#
# This code is hereby released into the public domain. It is released
# "as-is". All warranties disclaimed, implied or otherwise.
# 

# These are the external variables we need. This must be in a suitable
# directory.
unshift(@INC, "/usr/local/logger/sensor");
require "variables.ph";

`cp $LOGDIR/snif.date.1 $LOGDIR/snif.date.2`;
`cp $LOGDIR/snif.date.0 $LOGDIR/snif.date.1`;
`cp $LOGDIR/snif.date $LOGDIR/snif.date.0`;

# Current date goes into a file for later use
$tmp = sprintf("%02d%02d%02d%02d", @T[5],@T[4]+1,@T[3],@T[2]);
`echo "$tmp" > $LOGDIR/snif.date`;

if ( -f $LOGPID )
{
  $pid = `cat $LOGPID`;
  if ( $pid && kill (0, int($pid)) )
  {
    print "$0: $LOGPROG is already running, aborting\n";
    exit 1;
  }
}
#
# Prepare the parameters to pass to the tcpdump program.
#
$param = "$PROGPAR -w - -F $FILTER";

# Fire up the actual program. 
#
system("$LOGPROG $param 2>>$LOGDIR/tcpdump.err|$GZIPPROG > $TCPLOG 2>/dev/null&");

#
# Wait a couple of seconds for the chain to get started.
#
sleep 2;

# Since the gzip is tagged on at the end of the command, we have to
# go through hoops to get the right PID for the LOGPROG.
# NB! This is by no means bulletproof! You might find that once in a while,
# the ps listing will not contain the line you're looking for. Odd that.
#
# Note from Bill Ralph: this has been changed to be "SYSV" specific. The
# "ps -ef" works on Linux, so it can't be used to distinguish between BSD and
# SYSV. The following line works on Linux and SunOS 4.1.x. I recognize that
# this is very non-portable, if you have a more portable method, let me know.
#
$pid = `ps ax|grep $LOGPROG |awk '{print \$1}'`;
$pid =~ tr/\n/ /;

# Saves the new pid in the LOGPID file (hopefully we found it :-).
# It may be that more than one process fit the above description...
#
unlink $LOGPID;
`echo "$pid" > $LOGPID`;

# And we let them know if we didn't find it, as something serious
# could be wrong.

if ( ! $pid )
{
    print "$0: $LOGPROG may not be running, can't find pid in process table!\n";
    exit 1;
}
